package src.client;

import java.rmi.Naming;
import java.util.Date;
import src.common.*; 
 
public class Client {
	
	/* The Server and the port used is localhost, port:6000 */ 
	private static final String server = "127.0.0.1:6000";
	// Change this string if you wish to use a different port
	
	private String /* spec_public */ user, pass;
	// Username and pasword of the identity to be authenticated
			
	private static final String serverObject = "rmi://" + server + "/"
											+ "RemoteLoginServer";
	// Full adress and name of the remote object.
	
	private /* spec_public */ LoginInterface loginServer;
	// Interface to the login object that authenticates users
	
	private /* spec_public */ ServerInterface theServer;

	/**
	 * The ghost variable used to mark the state that 
	 * the program is currently in
	 */
	//@ private static ghost String state=null;
	
	/*@ invariant 
	  @		( state==INITIAL || state==CONNECTED || 
	  @  	  state==AUTHENTICATED || state==REQUEST_METHOD || 
	  @		  state == CREATE_NEW_LOGIN_CONTEXT || state==TERMINATE ); 
	  @*/
	
	/*@ constraint
	  @ 	( state==INITIAL ==> \old(state==null ) &&
	  @		( state==FOUND ==> \old(state)==INITIAL ) &&
	  @		( state==AUTHENTICATED ==> \old(state)==FOUND ) &&
	  @		( state==REQUEST_METHOD ==> \old(state)==AUTHENTICATED ) &&
	  @		( state==TERMINATE ==> \old(state)==INITIAL
	  @							  || \old(state)==FOUND
	  @							  || \old(state)==AUTHENTICATED
	  @							  || \old(state)==CREATE_NEW_LOGIN_CONTEXT
	  @							  || \old(state)==REQUEST_METHOD ));
	  @*/
	
	
	// 	CONSTRUCTOR
	/*@ requires	state==null;
	  @
	  @ ensures		state==INITIAL &&
	  @				loginServer==null && theServer==null && 
	  @				\only_called(start); 
	  @*/
	public Client() {
		//@ set state = INITIAL;
		loginServer = null;
		theServer = null;
		start();

	}											
	
	/**
	 * Looks up the reference to the remote object; if found attempts to
	 * athenticate the user and then calls methods for him. If any of the
	 * methods called by this method returns false it exits the programe.
	 */
	/*@ requires	state==INITIAL;
	  @
	  @ ensures		(lookupRemoteObj() && requestLogin() ==>
	  @							\only_called(performTask) ) &&
	  @	
	  @				(!lookupRemoteObj() || !requestLogin())
	  @			   				==> \only_called(exit))
	  @								
	  @*/
	private void start() {
		//@ set state = INITIAL;
		if (lookupRemoteObj()) {
			if (requestLogin()) {
				performTask();
			}
					
		} else {
			exit();
		}
	}

	/**
	 * Looks up the reference to the remote object on the server
	 */
	/*@ requires	state == INITIAL && serverObject != null
	  @							     && loginServer == null;
	  @
	  @ assignable	loginServer;
	  @
	  @ ensures		 \only_called(lookup) &&
	  @				 ( (loginServer==null ==> result==false) &&
	  @			
	  @				   (loginServer!=null ==> result==true) &&
	  @				    \fresh(loginServer) );
	  @*/
	private boolean lookupRemoteObj() {
		try {
			loginServer = (LoginInterface) Naming.lookup(serverObject);
			
			//@ set state = FOUND;
						
		} catch (Exception e) {
			return false;
		}
	return true;
	}
	
	/**
	 * Attempt to login the user.
	 */
	/*@ requires	state == FOUND && loginServer != null;
	  @
	  @ assignable	user, pass, theServer;
	  @
	  @ ensures		 \only_called(forwardCrds) &&
	  @				 ( (theServer==null ==> result==false) &&
	  @			
	  @				   (theServer!=null ==> result==true) &&
	  @				    \fresh(theServer) );
	  @*/
	private boolean requestLogin() {
		user="admin1"; pass="admin1"; // USERNAME AND PASSWORD  
									  // YOU WANT TO AUTHENTICATE WITH
		// update these fields with credentials of the identity 
		// you want to authenticate
		try {
			
		  theServer=(ServerInterface) loginServer.forwardCrds(user,pass);
		  //@ set state = AUTHENTICATED;
		} catch (Exception e) {
			//@ assume state == CREATE_NEW_LOGIN_CONTEXT;
			return false;
		}
		return true;
	}
	
	/**
	 * Call a remote method for the (authenticated) user 
	 */
	/*@ requires	state==AUTHENTICATED && theServer!=null;
	  @ 
	  @ ensures		 \only_called(insert, exit);
	  @*/
	private void performTask() {
		try {
			//@ set state = REQUEST_METHOD;
			theServer.insert("cpr", "Stephen", "Nielsen", 
						"Elektrovej 330, Lyngby 2800", new Date(901205));
		} catch (Exception e) {
			//@ assume state==VERIFY_AUTHORIZATION;
			// User is not authorised for this method.
			exit();  // Exit the program
		} 
		exit(); 	// Method completed successfully. Exit program.
	}
	
	/*@ reqiures	true; 
	  @ ensures		state==TERMINATE;
	  @*/
	private /* spec_public */ void exit() {
		//@ set state = TERMINATE;
		System.exit(0);
	}
}
