package src.server;

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

import javax.security.auth.*;
import javax.security.auth.callback.*;
import javax.security.auth.login.*;

import src.common.LoginInterface;
import src.common.ServerInterface;

public class LoginImpl extends UnicastRemoteObject 
									implements LoginInterface {

	// The referece to the object in the server where functionality 
	// of the methods is implemented.
	private /* spec_public */ ServerInterface myServer;
	
	//@ private static ghost String state;
	
	private /* spec_public */ LoginContext lc;
	
	/*@ invariant 
	  @		state==CREATE_LOGIN_CONTEXT || 
	  @					state==NEW_SUBJECT || state==NEW_PROXY;				
	  @*/
	
	
	/*@ constraint
	  @ 	(state==NEW_SUBJECT ==> \old(state)==CREATE_LOGIN_CONTEXT) &&
	  @		(state==NEW_PROXY ==> \old(state)==NEW_SUBJECT) &&
	  @ 	(state==CREATE_LOGIN_CONTEXT ==> \old(lc)!=lc);
	  @*/
	
	// Constructor
	protected LoginImpl(ServerInterface theServer) throws RemoteException {
			myServer = theServer;
	} 

	/**
	 *  Allows the client to login by creating a proxy for him and  
	 *  returning him the reference so that he can call methods on it
	 */
	/*@	normal_behavior
	  @ 	requires	username != null && password != null;
	  @		assignable	lc, user;
	  @		ensures		\fresh(lc, subject);
	  @ also
	  @	  exceptional_behavior
	  @		requires	lc==null;
	  @		signals 	(LoginException le) 
	  @							le.getMessage()!=null && \fresh(le);
	  @ also
	  @	  exceptional_behavior
	  @	   requires	lc!=null || lc==null;
	  @		signals 	RemoteException;
	  @				
	  @*/
	public ServerInterface forwardCrds(String username, String password) 
								  throws RemoteException, LoginException {

		//@ assert state==FOUND;
		lc = new LoginContext("JAASLogin", 
							new RemoteCallbackHandler(username, password));

		//@ set state==CREATE_LOGIN_CONTEXT;		
		lc.login();
		
		Subject subject = lc.getSubject();
		//@ set state==NEW_SUBJECT;		
		
	// Return the reference of a proxy to the calling client.			
	return createNewProxy(subject);
	}
	
	/*@	normal_behavior
	  @		requires	subject!=null && myServer!=null; 
	  @ 	ensures		\result==(ServerProxy(subject, myServer));
	  @ also
	  @	  exceptional_behavior
	  @		requires	lc!=null;
	  @		signals 	RemoteException;
	  @*/
	private ServerProxy createNewProxy(Subject subject) 
											throws RemoteException {
		//@ set state==NEW_PROXY;
		return new ServerProxy(subject, myServer);
	}

}




	/**
	 * The inteface between the user input and the security mechanisms.
	 * It decouples the service provider from the specific input device 
	 * being used to enter credentials.
	 */

class RemoteCallbackHandler implements CallbackHandler {
	private String username;
	private String password;
	
	RemoteCallbackHandler(String username, String password){
		this.username = username;
		this.password = password;
	}
	public void handle(Callback[] cb) {
    	for (int i = 0; i < cb.length; i++){
			if (cb[i] instanceof NameCallback){
				NameCallback nc = (NameCallback)cb[i];
				nc.setName(username);
			} else if (cb[i] instanceof PasswordCallback){
				PasswordCallback pc = (PasswordCallback)cb[i];
				pc.setPassword(password.toCharArray());
				password = null;
			}
		}
	}
}
