package src.server;

import java.io.*;
import java.rmi.RemoteException;
import java.util.*;

import src.common.ServerInterface;

  /*
   * The class where the use cases are actually implemented.
   */

public class ServerImpl implements ServerInterface {

	private Patient P;
	// The patient class
	private MedRecord M;
	// The Class containing a single medical record entry for a patient
	private ArrayList medicalRecords;
	// The list containing all medical records of a single Patient
	
	private Hashtable patientList;
	// Maps CPR-ids to the patient object 
	
	private HashMap patientRecords;
	// Maps Patient CPR-ids to the ArrayList containing his records
		
	
	// CONSTRUCTOR
	public ServerImpl() {
		patientList = new Hashtable();
		patientRecords = new HashMap();
	}
	
	/**
	 * Check if this patient ID exists, if it doesnt; insert a new patient 
	 * in the patient list
	 */
	public boolean insert(String cpr, String fName, 
				String lName, String adress, Date DOB) 
					throws RemoteException, SecurityException {
		
		if (patientList.containsKey(cpr)==false) {
			// Patient does not exist. Insert it
			patientList.put(cpr, new Patient(fName, lName, adress, DOB));
			return true;
			} else {
				return false;
				// This Patient already exists. Do nothing.
			}
				
	}
	
	/**
	 * Looks for the patient registered with this cpr ID. 
	 * If found it returns the patient data. 
	 * If NOT found it returns null.
	 */
	public String findP(String cpr) throws RemoteException, 
											SecurityException {
		
		if (patientList.containsKey(cpr)) {
			P = (Patient) patientList.get(cpr);
			return P.toString();
		} else		
			return null;
		
	}
	/**
	 * Edits patients name, lastname or adress. Edits them only if the value 
	 * of their parameter is NOT an empty string. If it is an empty string 
	 * is given, then that field is not updated.
	 */
	public boolean editP(String cpr, String fName, 
						String lName, String adress) 
							throws RemoteException, SecurityException {
								
		// Check which of the fields has been provided by the caller 
		// to be update that field
		if (patientList.containsKey(cpr)){
			P = (Patient) patientList.get(cpr);
				if (fName!="")
					P.setfName(fName);
				if(lName!="")
					P.setlName(lName);
				if(adress!="")
					P.setAdress(adress);
			
			// Place the newly updated object into the hashtable
			patientList.put(cpr, P);
			return true;
		}
	// no user exists with this cpr ID.
	return false;	
	}
	
	/**
	 * Check if patient exists. If not, no data is added.
	 * If yes, check if he as previous records and append the newest record
	 * to his list, if no he as previous records then create a record
	 * list for him and append the newest record to his list.
	 * 
	 */
	public boolean addMedicalRecords(String cpr, Date date, ArrayList test, 
						String comment, Double totalCost) 
								throws RemoteException, SecurityException {
		
		if (patientList.containsKey(cpr)) {
		// This Patient exists in the list of patients
			
			// Check if he has a privious history
			if(patientRecords.containsKey(cpr)) {
				
				// Patient has already a history of records
				medicalRecords = (ArrayList) patientRecords.get(cpr);
				// Get his list of medical records			
				M = new MedRecord(date, test, comment, totalCost);
				// Create a new Record with the data recieved 
				// through parameters to add it to his previous records.
				medicalRecords.add(M);
				//Insert the newly created record in his list
				return true;
				
			} else { 
				// This is the first record being entered for this patient
				medicalRecords = new ArrayList();
				// Create an (empty) history for him
				M = new MedRecord(date, test, comment, totalCost);
				// Create a new Record with the data recieved 
				// through parameters
				medicalRecords.add(M);
				//Insert the newly created record in his list
				return true;
			}
		}
		return false;
		// This patient is not registered in our patients list thus, 
		// no records can be added for him.
	}

	/**
	 * List records of the patient with the given CPR ID number.
	 * Return NULL if patient does not exist
	 */
	public ArrayList listRecords(String cpr) throws RemoteException, 
													SecurityException {
		// Check if this patient ID exists
		if (patientList.containsKey(cpr)) {
			// This Patient exists
				medicalRecords = (ArrayList) patientRecords.get(cpr);
				// Get the history of Patient

			return medicalRecords;
			// Return the ArrayList with history objects
		}
		return null;
		// Patient does not exist
	}	
	
}
